/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>

/* (1) Define accelerator table */
static char accel_copy[]  = "#override  Ctrl<Key>c : ArmAndActivate()";
static char accel_cut[]   = "#override  Ctrl<Key>t : ArmAndActivate()";
static char accel_paste[] = "#override  Ctrl<Key>p : ArmAndActivate()";

/* (4) Copy the selected text to clipboard */
static void CopyCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget text = (Widget)client_data;
    XmTextCopy(text, XtLastTimestampProcessed(XtDisplay(text)));
}

/* (5) Cut the selected text to clipboard */
static void CutCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget text = (Widget)client_data;
    XmTextCut(text, XtLastTimestampProcessed(XtDisplay(text)));
}

/* (6) Paste the data in clipboard into text */
static void PasteCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget text = (Widget)client_data;
    XmTextPaste(text);
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget toplevel, panel, text, control, copy, cut, paste;
    Arg    al[20];
    int    ac;
    XtAccelerators accel_table;

    XtSetLanguageProc(NULL, NULL, NULL);
    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Accelerator", NULL, 0,
                               &argc, argv, NULL, al, ac);
    ac = 0;
    panel = XmCreateRowColumn(toplevel, "panel", al, ac);
    XtManageChild(panel);

    ac = 0;
    text = XmCreateText(panel, "text", al, ac);
    XtManageChild(text);

    ac = 0;
    control = XmCreateRowColumn(panel, "control", al, ac);
    XtManageChild(control);

    ac = 0;
    copy = XmCreatePushButton(control, "copy", al, ac);
    XtManageChild(copy);
    ac = 0;
    cut = XmCreatePushButton(control, "cut", al, ac);
    XtManageChild(cut);
    ac = 0;
    paste = XmCreatePushButton(control, "paste", al, ac);
    XtManageChild(paste);

    XtAddCallback(copy,  XmNactivateCallback, CopyCB,  text);
    XtAddCallback(cut,   XmNactivateCallback, CutCB,   text);
    XtAddCallback(paste, XmNactivateCallback, PasteCB, text);

    /* (2) Convert accelerator string into internal format and set to the action widget */
    accel_table = XtParseAcceleratorTable(accel_copy);
    XtVaSetValues(copy,  XmNaccelerators, accel_table, NULL);
    accel_table = XtParseAcceleratorTable(accel_cut);
    XtVaSetValues(cut,   XmNaccelerators, accel_table, NULL);
    accel_table = XtParseAcceleratorTable(accel_paste);
    XtVaSetValues(paste, XmNaccelerators, accel_table, NULL);

    /* (3) Install the accelerator to the text widget */
    XtInstallAccelerators(text, copy);
    XtInstallAccelerators(text, cut);
    XtInstallAccelerators(text, paste);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(app_context);
}
