/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <time.h>
#include <Xm/XmAll.h>

/* Function to be called when input occur */
static void Input(label, fd)
    Widget label;
    int    fd;
{
    char     buff[BUFSIZ];
    int      len;
    XmString labelstr;

    len = read(fd, buff, BUFSIZ); /* input from file descriptor */
    if (len < 1) /* return on error */
        return;
    if (buff[len-1] == '\n') buff[len-1] = NULL;  /* ignore last new line */

    /* Create compound string and show it on the label */
    labelstr = XmStringCreateLocalized(buff);
    XtVaSetValues(label, XmNlabelString, labelstr, NULL);
    XmStringFree(labelstr);  /* Free the unnecessary compound string */
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget  toplevel, panel, field, title, label, button;
    Arg     al[20];
    int     ac;
    FILE   *fp;
    int     fd;
    int     readmask, readfds;
    struct timeval timeout;

    XtSetLanguageProc(NULL, NULL, NULL);
    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Inputselect", NULL, 0,
                               &argc, argv, NULL, al, ac);
    ac = 0;
    panel = XmCreateRowColumn(toplevel, "panel", al, ac);
    XtManageChild(panel);
    ac = 0;
    field = XmCreateRowColumn(panel, "field", al, ac);
    XtManageChild(field);
    ac = 0;
    title = XmCreateLabel(field, "title", al, ac);
    XtManageChild(title);
    ac = 0;
    label = XmCreateLabel(field, "label", al, ac);
    XtManageChild(label);
    ac = 0;
    button = XmCreatePushButton(panel, "button", al, ac);
    XtManageChild(button);

    fp = popen("printrandom", "r");  /* Invoke other process */
    if (fp == NULL) {
        fprintf(stderr, "Cannot start sub program.\n");
        exit(1);
    }
    fd = fileno(fp);    /* File descriptor of pipe (fp) */

    timeout.tv_sec = 0;     /* set the timeout for select() */
    timeout.tv_usec = 100;
    readmask = 1 << (fd);   /* set the mask for select() */

    XtRealizeWidget(toplevel);

    /* Custom event loop */
    while (True) {
        XEvent event;

        /* If X event is arrived, retrieve and process it */
        if (XtAppPending(app_context)) {
            XtAppNextEvent(app_context, &event);
            XtDispatchEvent(&event);
        }

        readfds = readmask;  /* reset the mask for select() */
        /* If there is input from pipe, call Input() */
        if ( (select(fd + 1, &readfds, NULL, NULL, &timeout) > 0)
              && (readfds & readmask) )
            Input(label, fd);
    }
}
